/* nobsdes6.ado -- hst 06.03.2020 / 30.06.2020 / 28.01.2021 / 06.01.2022 / 17.02.2022 / 29.04.2022 / 02.06.2022 */

/* nobsdes6.ado  ermittelt fuer mehrere Variablen die Anzahl der unterschiedlichen 
   IDs mit validen Beobachtungen. Benoetigt wird der Name der ID und der 
   interessierenden Variablen (in dieser Reihenfolge). 
   Die Optionen "histogram", "pctile" und "byest" werden in eine "by"-Option 
   transformiert, wobei bei "histogram", "pctile" und "byest" die kleinste und die 
   größte Gruppe dem allgemeinen Schema folgen, während die übrigen abweichend 
   behandelt werden, da die Intervallgrenzen Externen grundsätzlich nicht 
   bekannt sind. Diese Abweichung wird mit der lokalen Variablen weakpctile 
   abgebildet. 
   Da in diesen Fällen für jede Variable unterschiedliche "by"-Optionen 
   resultieren, muss jede Variable einzeln untersucht werden. Es können nicht
   mehrere Variablen mit einer einheitlichen "by"-Option verdichtet werden. */

*! version 1.0.0  28.01.2021
*! - option `histogram' is now available. Syntax:
*! version 1.0.1  13.06.2021
*! - variable labels in output added
*! version 1.0.2  17.11.2021
*! - in case of a break, previously the variables nobs, CR2, pctile, and value
*! - remained in the dataset. Now, these are dropped
*! version 2.0.0  07.01.2022
*! - allows in most cases varlist, e.g.  nobsdes5 id x xpc, by(country)
*!   or nobsdes5 id x xpc, histogram(25 10)
*! - runs under STATA 15 as well as under STATA 16 and STATA 17
*! - part of code moved to subroutines 
*! version 2.0.1  10.01.2022
*! - error with `by' in output corrected `tg_id_by'==1 instead of `by'!=`by'[_n-1]
*! version 2.0.2  14.01.2022
*! - error with `by' in output corrected `tg_id_by'==1 instead of `by'!=`by'[_n-1]
*! version 2.0.3  17.02.2022
*! - error message in case of dummy variable in varlist
*! version 2.1.0  29.04.2022
*! - error with varlist if "crosstabs": within the loop all rows with missings 
*!   (and zeros) were deleted. Thus the data frame became smaller and smaller.
*!   A conservative error because the number of observations became smaller and
*!   smaller.
*! version 2.1.1  02.06.2022
*! - previous correction induced a new error with the effect that not at least
*!   five distinct entitites were requiered but two, as with nobsdes. This has
*!   been fixed.
*! - the calculations for pctile were incorrect. Instead of the n-th observation
*!   the n+1-th observation was taken as percentile.
*! - The message "Zero values for var found" is not displayed anymore with the
*!   pctile option.
*! - using the option 'generate' left a variable called 'rdscfrnobs' behind.
*!   This variable is now dropped after execution.
*! version 2.1.2  01.07.2022
*! - if there is a missing value in `by' a warning occurs
*! - previously, if a variable label was left blank [|] was displayed in the ouput
*! - in case of for example nobsdes5 id x1 x2 x3, miss(0) without further options
*!   the output is less repetitive than before


capture program drop nobsdes6
program define nobsdes6, rclass
	local vs = c(stata_version)
	local vu = c(userversion)
	version `vs'
	
	syntax [anything(name=clist)] [if] [in] [, BY(varlist) BYEST(varlist)  /*
	    */ BYREGIO(varlist) BYDUPL(varlist) MISS(string) GENerate Name(string)  /*
		*/ REPLACE NOTABle PCTILE(string) HISTogram(string) ENDmark(string) SHOW]

	local osor : sortedby
	local _minobs 6

	local anzvar : word count `clist'		
	tokenize  `clist'
	
	tempname  touse 
	mark     `touse' `if' `in'

	/* --------------------------   Command line   -------------------------- */	
	
	local commandline "nobsdes5 `clist'"
	if "`if'"!="" {
		local commandline "`commandline' `if'"
	}
	if "`in'"!="" {
		local commandline "`commandline' `in'"
	}
	if "`by'"!="" | "`byest'"!="" | "`byregio'"!="" | "`bydupl'"!="" | "`miss'"!="" | "`gen'"!="" | "`name'"!="" | "`replace'"!="" | "`notab'"!="" | "`pctile'"!="" | "`hist'"!="" {
		local options ","
	}
	if "`by'"!="" {
		local options "`options' by(`by')"
	}
	if "`byest'"!="" {
		local options "`options' byest(`byest')"
	}
	if "`byregio'"!="" {
		local options "`options' byregio(`byregio')"
	}
	if "`pctile'"!="" {
		local options "`options' pctile(`pctile')"
	}
	if "`hist'"!="" {
		local options "`options' hist(`hist')"
	}	
	if "`miss'"!="" {
		local options "`options' miss(`miss')"
	}
	if "`bydupl'"!="" {
		local options "`options' bydupl(`bydupl')"
	}
	if "`gen'"!="" {
		local options "`options' gen(`gen')"
	}
	if "`name'"!="" {
		local options "`options' name(`name')"
	}
	if "`notab'"!="" {
		local options "`options' notab)"
	}
	if "`replace'"!="" {
		local options "`options' replace(`replace')"
	}
	
	/* ---------------------------   Check input   -------------------------- */
	**# Check input
	if `anzvar'==1 & "`1'"!="BAID" & "`1'"!="BAID_DOM" & "`1'"!="BAID_MOF" & "`1'"!="BAID_FIL" & "`1'"!="BAID_CON" & "`1'"!="SYSNR" & "`1'"!="SYSNR_DOM" & "`1'"!="SYSNR_MOF" & "`1'"!="SYSNR_FIL" & "`1'"!="SYSNR_CON" & "`1'"!="AWMUS_CD" & "`1'"!="AWMUSID" & "`1'"!="BBK_HM_CO_ID1" {
		display as error "Warning" as text " Is " as smcl "{it:`1'}" _asis " an identifier?"
	}

	/* Is id string or numeric ? */
	capture confirm string variable `1'
	if _rc==0 {
		local   id `1'
	    quietly replace `touse' = 0 if `id'==""
		quietly count if `id'==""
		local Nmiss = r(N)
	}
	else if _rc!=0 {
		local   id "`1'"
	    quietly replace `touse' = 0 if `id'==.
		quietly count if `id'==.
		local Nmiss = r(N)		
	}
	if `Nmiss'!=0 {
		display as error "Warning" as text " There are missing values in identifier " as smcl "{it:`1'}" _asis "."	
	}
	
	/* Is identifier or variable name missing ? */
	if ("`byest'"!="" | "`histogram'"!="" | "`pctile'"!="") & `anzvar'<2 {
		display as error "You need an identifier and at least one further variable."
		exit
	}
	

	/* Are there missing values in `by' (numeric string) ? */
	if "`by'"!="" {	
		foreach var in `by' {
			capture confirm numeric variable `var'
			if !_rc {
				capture assert `var'!=.
				if _rc!=0 {
					display as error "Warning" as text " Missing values for variable " as smcl "{it:`var'}" _asis " found -> Do you want to restrict " as smcl "{bf:if `var'!=.}" _asis " ?"
				}
			}
			else {
				capture assert `var'!=""
				if _rc!=0 {
					display as error "Warning" as text " Missing values for variable " as smcl "{it:`var'}" _asis " found -> Do you want to restrict " as smcl "{bf:if `var'!=`""'}" _asis " ?"

				}
			}
		}
	}
	
	/* Setting kritical value for regional data */
	if "`byregio'" != "" {
		local by = "`byregio'"	
		local _minobs 20 
	}
	
	/* Does (dupl) show up in clist? Create varlist without (dupl) */
	if "`bydupl'"!="" {
		local varlist ""
		local i 1
		while `i' <= `anzvar' {
			local var : word `i' of `clist'	
			if "`var'" != "(dupl)" {
				local varlist `varlist' `var'
				local i = `i' + 1							
			}
			else {
				local i = `i' + 1  /* 2 */
			}
		}
	}
	else {
		local varlist `clist'
		local anzvar : word count `varlist'		
		tokenize  `varlist'
	}

	/* Does (log) show up in clist? Create varlist without (log). Set local log */
	local varlist2 ""
	local log ""
	local i 1
	while `i' <= `anzvar' {
		local var : word `i' of `varlist'	
		if "`var'" != "(log)" {
			local varlist2 `varlist2' `var'
			local i = `i' + 1							
		}
		else {
			local log "log"
			local i = `i' + 1  /* 2 */
		}
	}
	local varlist "`varlist2'"	
	
	/* New variable `name' which contains the number of distinct values of the 
	   varlist by levels defined by the categorical variable supplied in the 
	   by option. */
	if "`name'" == "" {
		tempvar   name 
	}	
	if "`replace'"!="" {
		capture drop `name'
	}	
    
	/* Is miss option requiered? Umsetzung der miss-Option erfolgt später */
	if `anzvar'>=2 {
		tokenize `varlist', parse(" ")
		macro shift
		local noidlist `*' 		
		foreach var in `noidlist' {
			capture assert `var'==0|`var'==1|`var'==.     /* Is dummy? */
			if _rc!=0 {
				capture assert `var'!=0
				if _rc!=0  & "`miss'"=="" & "`pctile'"=="" {		
					display as error "Warning" as text " Zero values for variable " as smcl "{it:`var'}" as text " found -> Should you use option " as smcl "{bf:miss(0)}" _asis " ?"
				}
			}
			else {
				display as smcl "{error}{it:`var'}" _asis " is a " as error "0-1 dummy variable. " as text "Dummy and categorial variables " as error "must not " as text "be part of varlist but " as error "must " as text "be included in by() -> " as smcl "{bf:by(`var')}" _asis
				exit
			}
		}
	}

	/* Generate extvar to manipulate extvar and labels */
	if `anzvar'==1 {
		local   extvar = 1
		local   kritcr = 0
	}
	
	
	if "`histogram'"!="" & `touse' {
		if "`show'"=="" {
			local notable = "notable"
		}
	}

	
	/* Inquiery, whether percentiles outside the admissible range are not sorted */
	if "`pctile'"!="" {	
		tokenize "`pctile'"
		local pctilelist `*'		
		local pctold = 0
		local hv = 0
		foreach pct in `pctilelist' {
			if "`pct'"!="|" {
				if `pct'<=0 | `pct'>100 {
					noi di as error "Percentile (`pct') has to be greater than 0 and smaller than 100"
					exit				
				}			
				if `pct'<=`pctold' & `hv'==0 {
					noisily {
						display as error "Percentiles not sorted:" as text " `pctold' is larger than `pct'"
					}
					exit
				}
				local pctold = `pct'
				local hv = 0
			}
			else if "`pct'"=="|" {
			    local hv = 1	
			}
		}	
	}
	
	/* Here, `name' is dummy variable for non missings tag() */	
	qui egen byte `name' = tag(`id' `by') if `touse' 

	if "`generate'"!="" & `anzvar'>=2 {
		foreach var in `noidlist' {	
			if "`replace'"!="" {
				cap drop tg_`id'_`var'
				cap drop cr2_`id'_`var'				
			}	
			qui generate byte  tg_`id'_`var' = `name'
			qui replace        tg_`id'_`var' = 0       if  `var'==.
		}
	}
	else if "`generate'"!="" & `anzvar'==1 {
		if "`replace'"!="" {
			cap drop tg_`id'
		}	
		qui generate byte tg_`id' = `name'	
	}

	if "`show'"!="" {
		noi di "Input check passed"
	}

	
    /* ---------------------------   Evaluations   -------------------------- */
	**# Evaluations
	
	/* Calculate concentration ratio for the two largest entities CR2 */
	/* was wird weiterverwendet? */

	**# histogram
	if "`histogram'"!="" {
		local anzvar : word count `varlist'
		local anzhis : word count `histogram'
		qui forvalues i = 2/`anzvar' {
			local extvar : word `i' of `varlist'	
			if `anzhis'==1 {
				local hist = `histogram'						
			}
			else {
				local j = `i'-1				
				local hist : word `j' of `histogram'	
			}
			preserve
			if "`miss'"!="" {
				replace `extvar' = . if `extvar'==`miss'
			}
			drop if   `touse'==0
			tempvar    binvar        /* binvar   = bins */
			summarize `extvar'	
			generate  `binvar'   = 1 + int(`hist'*(`extvar'-r(min))/(r(max)-r(min) ))  if `extvar'!=.
			replace   `binvar'   = `binvar' - 1 if `binvar'==`hist'+1
			local labextvar : variable label `extvar'			
			collapse (sum) `extvar', by(`id' `by' `binvar')
			tempvar   id_tot minshare tot share name r CR2
			egen      `id_tot'   = total(`extvar'),				by(`id')           
			egen      `minshare' = min(`extvar'/`id_tot'*100),  by(`by' `binvar') 
			egen      `tot'      = total(`extvar'),				by(`by' `binvar') 
			generate  `share'    = `extvar'/`tot'*100                 
			replace   `share'    = 0 if `tot'==0	
			egen      `name'     = total(`extvar'!=0),			by(`by' `binvar') 	
			gsort `by' -`share'
			if "`by'"=="" {
				generate `r'   = sum(1)           if `name'>0
				egen     `CR2' = total(`share')	  if `r'<=2, by(`binvar')
			}
			else {
				by `by': generate `r' = sum(1)           if `name'>0
				egen     `CR2'		  = total(`share')	 if `r'<=2, by(`by' `binvar')
			}			
			
			*egen		`CR2'	 = total(`share')	if `r'<=2,	by(`by' `binvar') 
			format %8.0f `CR2'		
			label variable `name'      "nobs"
			label variable `CR2'       "CR2"
			label variable `minshare'  "minshare"
			replace   `name'     = . if `r'!=1  
			drop if   `r'!=1

			if "`show'"!="" {
				noi di "Step 3 : histogram calculations for `extvar' passed"
				noi table `by' `binvar',  statistic(mean `name' `CR2' `minshare') nototal nformat("%8.0f")							
			}
			
			/* ------   Report (message) in case of histogram option   ------ */

			/* Zwischen kleinstem und größtem Quantil (`binvar'>1&`pcmax'==0), 
			Fallzahl zu gering (`totname'> 0 & `totname'< `_minobs') und 
			Dominanzkriterium verletzt (`minshare'>85) */		
			
			/* weakpctile und kritischer Wert für CR2 (kritcr) und für nobs (kritnobs) */
			tempname inbetween border interior

			generate  `border'    = 0
			generate  `interior'  = 0
			generate  `inbetween' = 0
			rmin_prog `inbetween' `name' `CR2' `binvar' `border' `interior' `minshare', minobs(`_minobs') 
			local kritcr     = r(kritcr)
			local kritcr     = abs(`kritcr')			
			local kritnobs   = r(kritnobs)
			local weakpctile = r(weakpctile)	
			local rmin 		 = r(rmin)

			noi {
			display ""
			display as text "Output control for variable " as smcl "{it:`extvar' [`labextvar']}"	_asis
			if (`kritnobs'<`_minobs' & `kritnobs'>0)|(`kritcr'>85 & `kritcr'<=100)|`weakpctile'==1 {	
				display as error "D I S C L O S U R E problem: "
			}					
			if `kritcr'>85 & `kritcr'<=100 {
				display as text "Share of largest two IDs > 85% and <= 100%" 
			}				
			else if `kritcr'<=85 & `weakpctile'==0 {
				display as text "No problem with dominance " 
			}
			if `weakpctile'==1 {
				display as text "Less than 15 percent of variable " as smcl "{it:`extvar'}" _asis " of at least one ID (" as smcl "{it:`id'}" _asis ") outside a bin with less than `_minobs' distinct IDs"
				* capture drop bin
				rename `binvar' bin
				list    bin if `interior'==1, noobs	
				rename  bin `binvar'
			}
			display as text "Number of distinct IDs (" as smcl "{it:`id'}" _asis ") for the smallest or largest of `hist' bins of variable " as smcl "{it:`extvar'}" _asis " : `rmin'"
			display ""	
			}
			restore
		}
		if "`show'"!="" {
			noi di "Step 4 histogram completed"
		}
	}

	**# byest
	else if "`byest'"!="" {
		local anzvar : word count `varlist'
		local anzest : word count `byest'
		qui forvalues i = 2/`anzvar' {
			local j = `i'-1
			local extvar : word `i' of `varlist'	
			local est    : word `j' of `byest'	
			preserve
			if "`miss'"!="" {
				replace `extvar'=. if `extvar'==`miss'
			}	
			drop if   `touse'==0			
			local labextvar : variable label `extvar'			
			collapse (sum) `extvar', by(`id' `by' `est')
			tempvar   id_tot minshare tot share name r CR2
			egen      `id_tot'   = total(`extvar'),             by(`id')        
			egen      `minshare' = min(`extvar'/`id_tot'*100),	by(`by' `est')  
			egen      `tot'      = total(`extvar'),             by(`by' `est') 
			generate  `share'    = `extvar'/`tot'*100           
			replace   `share'    = 0 if `tot'==0	
			egen      `name'     = total(`extvar'!=0),          by(`by' `est') 	
			gsort     `by' `est' -`share'	
			local BY `"by `by' `est':"'
			`BY' generate `r'    = sum(1)           if `name'>0
			egen		`CR2'	 = total(`share')	if `r'<=2,  by(`by' `est') 
			format %8.0f `CR2'	
			label variable `name'      "nobs"
			label variable `CR2'       "CR2"
			label variable `minshare'  "minshare"
			replace   `name'     = . if `r'!=1  
			drop if `r'!=1
			
			if "`show'"!="" {
				noi di "Step 3 byest calculations for `extvar' passed"
				noi table `by' `est',  c(mean `name' mean `CR2' mean `minshare') 				
			}

			/* --------   Report (message) in case of byest option   -------- */

			/* Zwischen kleinstem und größtem Quantil (`byest'>1&`pcmax'==0), 
			Fallzahl zu gering (`name'> 0 & `name'< `_minobs') und 
			Dominanzkriterium verletzt (`minshare'>85) */
						
			/* weakpctile und kritischer Wert für CR2 (kritcr) und für nobs (kritnobs) */
			tempname inbetween border interior

			generate  `border'    = 0
			generate  `interior'  = 0
			generate  `inbetween' = 0
			rmin_prog `inbetween' `name' `CR2' `est' `border' `interior' `minshare', minobs(`_minobs') 
			local kritcr     = r(kritcr)
			local kritcr = abs(`kritcr')
			local kritnobs   = r(kritnobs)
			local weakpctile = r(weakpctile)	
			local rmin 		 = r(rmin)

			noi {
			display ""
			display as text "Output control for variable  " as smcl "{it:`extvar' [`labextvar']}" _asis 
			
			if (`kritnobs'<`_minobs' & `kritnobs'>0) | (`kritcr'>85 & `kritcr'<=100) {	
				display as error "D I S C L O S U R E problem: "	
			}
	
			if (`kritcr'>85 & `kritcr'<=100) {
				display as text "Share of largest two IDs > 85% and <=100%" 
			}				
			else if `kritcr'<=85 & `weakpctile'==0 {
				display as text "No problem with dominance " 
			}	
		
			local anzby : word count `est'
			if `anzby'==1 {	
				local labby : variable label `est'
				if length("`labby'") > 25 {	
					local labby = "[" + substr("`labby'",1,25) + " ..]"
				}
				else if length("`labby'") <= 25 & length("`labby'")!=0 {
					local labby = "[`labby']"
				}
			}
			if `weakpctile'==1 {
				display as text "Less than 15 percent of variable " as smcl "{it:`extvar'}" _asis " of at least one ID (" as smcl "{it:`id'}" _asis ") outside a bin (" as smcl "{it:`est'}" _asis ") with less than `_minobs' distinct IDs"
				list `est' if `inbetween'==1, noobs	
				display ""
			}		
		
			if "`notable'"!="" {
				display as text "Smallest number of distinct IDs (" as smcl "{it:`id'}" _asis ") of variable " as smcl "{it:`extvar'}" _asis " for " as smcl "{it:`est'}" _asis ": `rmin'"
			}
			else if "`notable'"=="" {
				display as text "Number of distinct IDs ("  as smcl "{it:`id'}" _asis ") of variable " as smcl "{it:`extvar'}" _asis " for each " as smcl "{it:`est'}" _asis " :"
				if (`kritcr'>85 & `kritcr'<=100) | (`kritnobs'>0 & `kritnobs'<`_minobs') { 
					nobreak {
						rename `CR2' CR2 
						rename `name' nobs							
						list   `est' nobs CR2 if `border'==1 & nobs!=. & `est'!=`est'[_n-1], noobs sep(0)  abb(15)				
						rename  CR2 `CR2'
						rename nobs `name' 				
					}
				}
				if `kritcr'<=85 & `weakpctile'==0 & `kritnobs'>=`_minobs' {
					nobreak {
						rename `name' nobs							
						list   `est' nobs     if `border'==1 & nobs!=. & `est'!=`est'[_n-1], noobs sep(0) abb(15)
						rename nobs `name' 				
					}
				}
			}
			}
			
			restore
		}
		if "`show'"!="" {
			noi di "Step 4 byest completed"
		}
	}
**# Bookmark #1
	
	else if "`pctile'"!="" & `anzvar'>=2 {
		local anzvar : word count `clist'
		local i 2
		local j 2
		* data frame allows list of several variables
		* mypctile allows only one variable, loop
		qui while `j' <= `anzvar' {
			local extvar : word `j' of `clist'
			/* anders hochzählen extvar_`i' : word `j' of `clist', Dopplung */
			if "`bydupl'"!="" {	
				tempvar extvar_nvalid ndupl		
				if "`extvar'" != "(dupl)" {

				}
				else if "`extvar'" == "(dupl)" {		
					local j = `j' + 1				
					local extvar : word `j' of `clist'	
					generate `extvar_nvalid' = (`extvar'!=.)						
					egen    `ndupl'      = total(`extvar_nvalid'), by(`bydupl')
					if "`show'"!="" {
						noi di "Step 3: `extvar'    ndupl"
						noi table `bydupl', statistic(mean `ndupl') nototals
					}
				}
			}				
			else {
				tempvar  ndupl
				generate `ndupl' = 1 
			}
			
			tokenize "`pctile'", parse("|")	
			local pctile1 `1'
			if "`2'"=="|" {
				macro shift 2
				local pctile `*'
			}
			capture drop `extvar'_pgr
			capture drop `extvar'_pgrstr			
			generate `extvar'_pgr 	  = 0   if `extvar'!=.  /* Benennung der Quantile */		
			generate `extvar'_pgrstr  = 0	if `extvar'!=.  
			tempvar  extvar1 
			generate `extvar1' = `extvar'
			preserve
			drop if `extvar'==. | `touse'==0
			if "`miss'"!="" {
				drop if `extvar'==`miss'
			}	
			
            * mypctile calculates the empirical values for `extvar'_pgr and `extvar'_pgrstr [weight=`ndupl']
	        mypctile `id' `extvar1' `extvar'_pgr `extvar'_pgrstr   if `extvar1'!=. & `touse', pctile("`pctile1'") by(`by') bydupl(`bydupl') 
			local pstring = r(pstring)
			local labextvar : variable label `extvar'						
		
			collapse (sum) `extvar1' `extvar' (mean) `extvar'_pgrstr  if `extvar1'!=., by(`id' `by' `extvar'_pgr) 
			sort `extvar'_pgr, stable
			tempvar    id_tot minshare tot share name r CR2
			egen      `id_tot'   = total(`extvar1'),              by(`id')  
			egen      `minshare' = min(`extvar1'/`id_tot'*100),   by(`by' `extvar'_pgr) 
			egen      `tot'      = total(`extvar1'),              by(`by' `extvar'_pgr) 
			generate  `share'    = `extvar1'/`tot'*100        
			replace   `share'    = 0 if `tot'==0	
			egen      `name'     = total(1),   			          by(`by' `extvar'_pgr) 	
			gsort     `by' `extvar'_pgr -`share'	
 		    local BY `"by `by' `extvar'_pgr:"'
  	        `BY' generate `r' = sum(1)      if `name'>0
			egen      `CR2'		 = total(`share')	 if `r'<=2,  by(`by' `extvar'_pgr)
			format %8.0f   `CR2'								
			label variable `name'      "nobs"
			label variable `CR2'       "CR2"
			label variable `minshare'  "minshare"
			sort `by' `extvar'_pgr `id' 
			if "`show'"!="" {
				noi di "Step 3 (pctile): `extvar1'"
				noi display "       by   extvar       tot     id       pgr     pgrstr   minshare      count        CR2      share "					
				noi list `by' `extvar1' `tot' `id'  `extvar'_pgr  `extvar'_pgrstr  `minshare' `name' `CR2' `share', sepby(`by')		
				noi table `extvar'_pgr, c(mean `name' mean `CR2' mean `minshare') format("%8.0f")
			}
			
			/* -------------------------------------------------------------- */
			
			/* Zwischen kleinstem und größtem Quantil (`binvar'>1 & `pcmax'==0), 
			Fallzahl zu gering (`totname'> 0 & `totname'< `_minobs') und 
			Dominanzkriterium verletzt (`minshare'>85) */		
			noi mess_pctile `id' `extvar' `CR2' `name' `extvar'_pgr `extvar'_pgrstr `minshare', `notable' minobs(`_minobs') labelby(`labextvar') by(`by') pstring("`pstring'") pctile("`pctile1'") 
			restore
			local j = `j'+1
		}
		if "`show'"!="" {
			noi di "Step 4 (pctile)"
		}
	}
	
	else if ("`pctile'"=="" & "`histogram'"=="" & "`byest'"=="") & `anzvar'>=2 {	
		preserve
		keep `id' `varlist' `by' `touse'
		qui keep if `touse'
			
		local intlist = ""
		local mislist = ""		
		local lablist = ""		
		local anzvar : word count `varlist'
		forvalues i = 2/`anzvar' {
			tempvar extvar_`i'_miss		
			local extvar_`i' : word `i' of `varlist'
			if "`miss'"!="" {
				qui replace `extvar_`i''=. if `extvar_`i''==`miss'
			}
			generate `extvar_`i'_miss' = (`extvar_`i''==.)	
			local labextvar: variable label `extvar_`i''
			local labextvar = "`labextvar' | "		
				
			local intlist `intlist' `extvar_`i''	
			local mislist `mislist' `extvar_`i'_miss'								
			local lablist `lablist' `labextvar' 					
		}	
		collapse (sum) `intlist' (mean) `mislist', by(`id' `by')
		local anzvar : word count `intlist'	
		qui if "`generate'" != "" {
			tempfile nobsgen
			save `nobsgen', replace
		}
		qui forvalues i = 1/`anzvar' {
			tempvar   id_tot shareid minshare tot share r CR2 name
			local     extvar : word `i' of `intlist'
			local     extvar_miss : word `i' of `mislist'
			tokenize "`lablist'", parse("|")	
			local labextvar `1'
			if "`2'"=="|" {
				macro shift 2
				local lablist `*'
			}
			label variable `extvar' "`labextvar'"
			egen      `id_tot'   = total(`extvar'), by(`id')
			egen      `tot'      = total(`extvar'), by(`by')
			generate  `share'    = `extvar'/`tot'*100       
			replace   `share'    = 0 if `tot'==0	
			if "`miss'"!="" {
				egen  `name'     = total(`extvar'!=. & `extvar'!=`miss'), by(`by') 	
			}
			else {
				egen  `name'     = total(`extvar'!=.), by(`by')								
			}
			
			gsort `by' -`share'
			if "`by'"=="" {
				generate `r'   = sum(1)           if `name'>0
				egen     `CR2' = total(`share')	  if `r'<=2
			}
			else {
				by `by': generate `r' = sum(1)           if `name'>0
				egen     `CR2'		  = total(`share')	 if `r'<=2, by(`by')
			}
			format %8.0f `CR2'								
			replace   `name'  	 = . if `r'!=1			
			qui summarize `CR2'			
			local kritcr   = abs(r(max))
			qui summarize `name'
			local kritnobs = r(min)		
			
			if "`show'"!="" {
				noi di "Step 3:  `vs'; `extvar'"
			}
			
			if `anzvar'>=1 & "`by'"=="" {
				noi mess_by `id' `extvar' `name' `CR2', `notable' minobs(`_minobs') labelby(`labextvar')  
				qui if "`generate'" != "" {
					egen         cr2_`id'_`extvar' = mean(`CR2'),  by(`by') 
					format %8.0f cr2_`id'_`extvar'							
					merge m:1   `id' using `nobsgen', nogen
					save        `nobsgen', replace
				}
			}	
			qui else if `anzvar'>=1 & "`by'"!="" {
				noi display ""
				noi mess_by `id' `extvar' `name' `CR2', `notable' minobs(`_minobs') labelby(`labextvar') by(`by')		
				qui if "`generate'" != "" {
					egen         cr2_`id'_`extvar' = mean(`CR2'),  by(`by') 		
					format %8.0f cr2_`id'_`extvar'												
					merge 1:1   `id' `by' using `nobsgen', nogen
					save        `nobsgen', replace
				}
			}
		}
		restore
		qui if "`generate'" != "" {
			if "`by'"=="" {
				merge m:1 `id' using `nobsgen', keepusing(cr2_`id'_*) nogen
			}
			else if "`by'"!="" {			
				merge m:1 `id' `by' using `nobsgen', keepusing(cr2_`id'_*) nogen			
			}
		}		
		if "`show'"!="" {
			noi di "Step 5"
		}			
	}
	capture drop nobs

	if "`show'"!="" {
		noi di "Step 6"
	}
	
	local anzvar : word count `varlist'	
	if `anzvar'==1 {		
		if  "`by'"=="" & "`byest'"=="" {
			qui count if `name'==1
			local kritnobs = r(N)	
			if (`kritnobs'<`_minobs' & `kritnobs'>0) {	
				display as error "D I S C L O S U R E problem: "
				local rmin "too small"
			}
			else {
				local rmin = `kritnobs'
			}		
			display as text "Number of distinct values for identifier  `id' :   `rmin'"
		}
		else if "`by'"!="" | "`byest'"!="" {
			local anzby : word count `by'
			if `anzby'==1 {	
				local labby : variable label `by'
				if length("`labby'") > 25 {	
					local labby = "[" + substr("`labby'",1,25) + " ..]"
				}
				else if length("`labby'") <= 25 & length("`labby'")!=0 {
					local labby = "[`labby']"
				}
			}
			tempvar hv
			qui egen      `hv' = total(`name') if `touse', by(`by')			
			qui summarize `hv'                 if `hv'!=0			
			local kritnobs = r(min)	
			display as text "Output control for variable  `by' [`labby']"	
			if (`kritnobs'<`_minobs' & `kritnobs'>0) {
				display as error "D I S C L O S U R E problem: "				
				local rmin "too small"
			}
			else {
				local rmin = `kritnobs'
			}		
			if "`notable'"!="" {
				display as text "Smallest number of distinct values for identifier `id' for `by' `labby' : 	`rmin'"
			}
			else if "`notable'"=="" {
				display as text "Number of distinct values for identifier `id' for each `by' :"	
				nobreak {
					sort   `by'
					rename `hv' nobs
					tempvar tg_id_by
					egen   `tg_id_by' = tag(`by') if nobs!=.
					list   `by' nobs              if `tg_id_by'==1, noobs sep(0)
					rename  nobs `hv'
				}
			}
		}
	}

	
	
	if "$endmark"!="" {
		local endtext "$endmark"
	}		
	if "`endmark'"!="" {
		local endtext = "`endmark'"
	}
	if "`endmark'"!="" | "$endmark"!="" {
		di "`endtext'"
	}
	
	if "`osor'" != "" {
		sort `osor'
	}
	capture label drop pgrforerr 
	version `vu'	
end		

* Report (message) in case of by option 
capture program drop mess_by
program define mess_by, rclass
	syntax [varlist] [if] [in] [, BY(varlist) BYREGIO(varlist) NOTABle SHOW /*
	    */ MINOBS(int 5) LABELBY(string)]

	tokenize `varlist'
	local id        "`1'"
	local extvar    "`2'"
	local name      "`3'"
	local CR2       "`4'"
    local labextvar	"`labelby'"

	tempname touse
	mark `touse' `if' `in'

	qui summarize `CR2'
	local kritcr = r(max)	
	qui summarize `name'
	local kritnobs = r(min)	
	if (`kritnobs'<`minobs' & `kritnobs'>0) {
		local rmin "too small"
	}
	else {
		local rmin = `kritnobs'
	}
	
	display ""
	display as text "Output control for variable  `extvar' [`labelby']"	
	if (`kritnobs'<`minobs' & `kritnobs'>0) | `kritcr'>85 {	
		display as error "D I S C L O S U R E problem: "	
	}
	if `kritcr'>85 {
		display as text "Share of largest two IDs > 85% " 
	}				
	else if `kritcr'<=85 {
		display as text "No problem with dominance " 
	}

	if "`by'"=="" {
		display as text "Number of distinct IDs (`id') of variable  `extvar' :   `rmin'"
	}
	else if "`by'"!="" {			
		local anzby : word count `by'
		if `anzby'==1 {	
			local labby : variable label `by'
			if length("`labby'") > 25 {	
				local labby = "[" + substr("`labby'",1,25) + " ..]"
			}
			else if length("`labby'") <= 25 & length("`labby'")!=0 {
				local labby = "[`labby']"
			}
		}
		
		if "`notable'"!="" {
		    display as text "Smallest number of distinct IDs (`id') of variable `extvar' for `by' `labby': `rmin'"
		}
		else if "`notable'"=="" {
			display as text "Number of distinct IDs (`id') of variable `extvar' for each `by' :"

			if `kritcr'>85 | (`kritnobs'>0 & `kritnobs'<`minobs') { 
				nobreak {
					rename `CR2' CR2 
					rename `name' nobs								
					sort   `by' 
					tempvar tg_id_by
					egen   `tg_id_by' = tag(`by') if nobs!=.
					list   `by' nobs CR2 if nobs!=. & `tg_id_by'==1, noobs sep(0)  abb(15)						
					rename  CR2 `CR2'
					rename nobs `name' 										
				}
			}
			if `kritcr'<=85 & `kritnobs'>=`minobs' {
				nobreak {
					rename `name' nobs								
					sort   `by'
					tempvar tg_id_by
					egen   `tg_id_by' = tag(`by') if nobs!=.
					list   `by' nobs              if `tg_id_by'==1, noobs sep(0) abb(15)
					rename nobs `name' 					
				}
			}
		}
	}
end	
	
* Report (message) in case of byest option 
capture program drop mess_byest
 program define mess_byest, rclass
	syntax [varlist] [if] [in] [, BY(varlist) BYREGIO(varlist) /*
	    */ MINOBS(int 5) NOTABle SHOW LABELBY(string)]

	tokenize `varlist'
	local id        "`1'"
	local extvar    "`2'"
	local est       "`3'"
	local name      "`4'"
	local CR2       "`5'"	
	local minshare  "`6'"
    local labextvar	"`labelby'"
 
 	/* weakpctile und kritischer Wert für CR2 (kritcr) und für nobs (kritnobs) */
	tempname inbetween border interior
	qui {
	generate  `border'    = 0
	generate  `interior'  = 0
	generate  `inbetween' = 0
	rmin_prog `inbetween' `name' `CR2' `est' `border' `interior' `minshare', minobs(`minobs') 
	local kritcr     = r(kritcr)
	local kritnobs   = r(kritnobs)
	local weakpctile = r(weakpctile)	
	local rmin 		 = r(rmin)
	}

	display ""
	display as text "Output control for variable  `extvar' [`labelby']"		
	if (`kritnobs'<`minobs' & `kritnobs'>0) | `kritcr'>85 {	
		display as error "D I S C L O S U R E problem: "	
	}
	
	if `kritcr'>85 {
		display as text "Share of largest two IDs > 85% " 
	}				
	else if `kritcr'<=85 & `weakpctile'==0 {
		display as text "No problem with dominance " 
	}	
		
	local anzby : word count `est'
	if `anzby'==1 {	
		local labby : variable label `est'
		if length("`labby'") > 25 {	
			local labby = "[" + substr("`labby'",1,25) + " ..]"
		}
		else if length("`labby'") <= 25 & length("`labby'")!=0 {
			local labby = "[`labby']"
		}
	}
	if `weakpctile'==1 {
		display as text "Less than 15 percent of variable `extvar' of at least one ID (`id') outside a bin with less than `minobs' distinct IDs"
		list `est' if `inbetween'==1, noobs	
		display ""
	}		
		
	if "`notable'"!="" {
	    display as text "Smallest number of distinct IDs (`id') of variable `extvar' for `est': `rmin'"
	}
	else if "`notable'"=="" {
		display "Number of distinct IDs (`id') of variable `extvar' for each `est' :"
		qui display "kritnobs  `kritnobs'"

		if `kritcr'>85 | (`kritnobs'>0 & `kritnobs'<`minobs') { 
			nobreak {
				rename `CR2' CR2 
				rename `name' nobs							
				list   `est' nobs CR2 if `border'==1 & nobs!=. & `est'!=`est'[_n-1], noobs sep(0)  abb(15)				
				rename  CR2 `CR2'
				rename nobs `name' 				
			}
		}
		if `kritcr'<=85 & `weakpctile'==0 & `kritnobs'>=`minobs' {
			nobreak {
				rename `name' nobs							
				list   `est' nobs     if `border'==1 & nobs!=. & `est'!=`est'[_n-1], noobs sep(0) abb(15)
				rename nobs `name' 				
			}
		}

	}

end	

* Report (message) in case of histogram option 	
capture program drop mess_histogram
program define mess_histogram, rclass
	syntax [varlist] [if] [in] [, BY(varlist) BYREGIO(varlist) /*
	    */ MISS(string) MINOBS(int 5) LABELBY(string) HISTogram(string) /*
		*/ NOTABle SHOW] 
	
	tokenize `varlist'
	local id       "`1'"
	local extvar   "`2'"
	local binvar   "`3'"
	local name     "`4'"
	local CR2      "`5'"	
	local minshare "`6'"

	/* weakpctile und kritischer Wert für CR2 (kritcr) und für nobs (kritnobs) */
	tempname inbetween border interior
	qui {
	generate  `border'    = 0
	generate  `interior'  = 0
	generate  `inbetween' = 0
	rmin_prog `inbetween' `name' `CR2' `binvar' `border' `interior' `minshare', minobs(`minobs') 
	local kritcr     = r(kritcr)
	local kritnobs   = r(kritnobs)
	local weakpctile = r(weakpctile)	
	local rmin 		 = r(rmin)
	}
						
	display ""
	display as text "Output control for variable  `extvar' [`labelby']"						
	if (`kritnobs'<`minobs' & `kritnobs'>0)|`kritcr'>85|`weakpctile'==1 {	
		display as error "D I S C L O S U R E problem: "
	}					
	if `kritcr'>85 {
		display as text "Share of largest two IDs > 85% " 
	}				
	else if `kritcr'<=85 & `weakpctile'==0 {
		display as text "No problem with dominance " 
	}
	if `weakpctile'==1 {
		display as text "Less than 15 percent of variable `extvar' of at least one ID (`id') outside a bin with less than `minobs' distinct IDs"
		* capture drop bin
		rename `binvar' bin
		list    bin if `interior'==1, noobs	
		rename  bin `binvar'
	}
    display as text "Number of distinct IDs (`id') for the smallest or largest of `histogram' bins of variable `extvar' : `rmin'"
	display ""	
end	
	
* Report (message) in case of pctile option 	
capture program drop mess_pctile
program define mess_pctile, rclass
	syntax [varlist] [if] [in] [, BY(varlist) MINOBS(int 5) PCTILE(string) /*
	    */ LABELBY(string) PSTRING(string) NOTABle SHOW]

	tokenize `varlist'
	local id        "`1'"
	local extvar    "`2'"
	local CR2	    "`3'"		
	local name      "`4'"	
	local pgr       "`5'"	
	local pgrhv     "`6'"	
	local minshare  "`7'"
	local pstr = subinstr("`pstring'"," ", "", .)
 
	tempname touse inbetween pgrstr border interior
	mark `touse' `if' `in'

	tokenize "`pctile'"
	local pctilelist `*'	
	local i = 1
	foreach pct in `pctilelist' {
		if `pct'>0 & `pct'<100 {		
			local i = `i' + 1
		}
		else if `pct'<=0 {
			local i = `i' + 1				
		}			
	}
	local psep = `i'		

	/* weakpctile und kritischer Wert für CR2 (kritcr) und für nobs (kritnobs) */	
	qui {
	generate  `border'    = 0
	generate  `interior'  = 0
	generate  `inbetween' = 0
	rmin_prog `inbetween' `name' `CR2' `pgr' `border' `interior' `minshare', minobs(`minobs') 
	local kritcr     = r(kritcr)
	local kritnobs   = r(kritnobs)
	local weakpctile = r(weakpctile)	
	local rmin 		 = r(rmin)
	generate  `pgrstr' = string(`pgrhv')  
	summarize `pgr'
	replace   `pgrstr' = "max" 	  if `pgr'==r(max)	
	}	
	display ""
	display as text "Output control for variable  `extvar' [`labelby']"
	if (`kritnobs'<`minobs' & `kritnobs'>0)|`kritcr'>85|`weakpctile'==1 {	
		noi display as error "D I S C L O S U R E problem: "
	}					
	
	if `kritcr'>85 {
		noi display as text "Share of largest two IDs > 85% " 
	}				
	else if `kritcr'<=85 & `weakpctile'==0 {
		noi display as text "No problem with dominance " 
	}

	if `weakpctile'==1 {
		noi display as text "Less than 15 percent of variable `extvar' at least one id (`id') outside a percentile (`pstr') with less than `minobs' distinct IDs"
		rename (`pgr' `pgrstr') (pctile value)
		sort    pctile
		noi list pctile value if `interior'==1 & `name'>0 & `name'<`minobs' & `minshare'>85 & value!="." & pctile!=pctile[_n-1], noobs	 abb(15)
		rename (pctile value) (`pgr' `pgrstr')
		display ""
	}
	if "`by'"=="" {
		if "`notable'"!="" {
			display as text "Number of distinct IDs (`id') for the smallest or largest percentile (`pstr') of variable `extvar' :  `rmin'"
		}
		else if "`notable'"=="" {
			if (`border'==1 & (`kritcr'>85 | (`kritnobs'>0 & `kritnobs'<`minobs'))) {
				sort    `pgr' `CR2' 
				display as text "Number of distinct IDs (`id') for the smallest and largest percentile of variable `extvar' :"					
				nobreak {
					rename (`pgr' `CR2' `name') (pctile CR2 nobs)
					sort pctile
					list pctile nobs CR2 if `extvar'!=. & `border'==1 & pctile!=pctile[_n-1] & CR2!=., noobs	 abb(15)					
					rename (pctile CR2 nobs) (`pgr' `CR2' `name')
				}
			}
			else if `border'==1 & `kritcr'<=85 & `kritnobs'>=`minobs' {
				sort    `pgr' `CR2' 
				display "Number of distinct IDs (`id') for each percentile (`pstring') of variable `extvar' `labextvar' :"				
				nobreak {
					rename  (`pgr' `pgrstr' `name')  (pctile value nobs)
					sort pctile	
					list pctile value nobs if nobs>`minobs' & nobs!=. & pctile!=pctile[_n-1], noobs  abb(15)
					rename  (pctile value nobs)  (`pgr' `pgrstr' `name') 
				}
			}
		}
	}
	else if "`by'"!="" {			
		local anzby : word count `by'
		if `anzby'==1 {	
			local labby : variable label `by'
			if length("`labby'") > 25 {	
				local labby = "[" + substr("`labby'",1,25) + " ..]"
			}
			else if length("`labby'") <= 25 & length("`labby'")!=0 {
				local labby = "[`labby']"
			}
		}
		if "`notable'"!="" {
			display as text "Number of distinct IDs (`id') for the smallest percentile (`pstr') of variable `extvar' for `by' :  `rmin'"
		}
		else if "`notable'"=="" {
			if (`border'==1 | `kritcr'>85 | (`kritnobs'>0 & `kritnobs'<`minobs')) {
				display "Number of distinct IDs (`id') for the smallest percentile (`pstr') of variable `extvar' for `by' : "
			}
			qui display "kritnobs  `kritnobs'    kritcr:  `kritcr'"
			if `kritcr'>85 | (`kritnobs'>0 & `kritnobs'<`minobs') { 
				nobreak {
					rename (`CR2' `name' `pgr') (CR2 nobs pctile)
					sort    `by' pctile	CR2
					list    `by' pctile nobs CR2 if nobs<`minobs' & nobs!=0 & pctile!=pctile[_n-1], sepby(`by') noobs  abb(15)
					rename (CR2 nobs pctile) (`CR2' `name' `pgr')
				}
			}
			else if `border'==1 & `kritcr'<=85 & `kritnobs'>=`minobs' {
				nobreak {
					rename  (`pgr' `pgrstr' `name')  (pctile value nobs)
					sort `by' pctile	
					list `by' pctile value nobs if nobs>`minobs' & nobs!=. & pctile!=pctile[_n-1], sepby(`by') noobs  abb(15)
					rename  (pctile value nobs)  (`pgr' `pgrstr' `name') 
				}
			}
		}
	}
end	
	

capture program drop rmin_prog
program define rmin_prog, rclass
	syntax [varlist(default=none)] [if] [in] [, MINOBS(int 5)]	

	tokenize `varlist'
	
	local inbetween   "`1'"
	local name        "`2'"
	local CR2         "`3'"
	local var         "`4'"
	local border      "`5'"
	local interior    "`6'"
	local minshare    "`7'"

	summarize `var' 
	replace   `border'    = (`var'==r(min)|`var'==r(max))
	replace   `interior'  = (`var'> r(min)&`var'< r(max))
	summarize `CR2'          if `border'==1
	return local kritcr   = r(max)
	replace   `inbetween' =      (`interior'==1 & `name'> 0 & `name'< `minobs' & `minshare'>85)
	summarize `name'  		 if   `border'  ==1 & `name'!=0  /* damit kritnobs für kleinstes und größtes Quantil ermittelbar */
	return local kritnobs = r(min)		
	if (r(min)>0 & r(min)<`minobs') {
		return local rmin = "too small"
	}
	else {
		return local rmin = r(min)
	}
	summarize `inbetween'
	return local weakpctile = r(max)			
end	

capture program drop lab
program define lab, rclass
	syntax [anything] [if] [in] [, MAXLET(int 25)]	

	local blank = " "
	local blank = `maxlet' * "`blank'"
	if length("`anything'") > `maxlet' {	
		return local labextvar = substr("`anything'",1,`maxlet') + " .."
	}
	else if length("`anything'") <= `maxlet' & length("`anything'")!=0 {
		return local labextvar = substr("`blank'",1,`maxlet'-length("`anything'")) + "`anything'"
	}
	else if length("`anything'")==0 {
		return local labextvar = "`blank'"
	}	
end


/* -----------------------   Calculates percentiles   ----------------------- */
capture program drop mypctile
program define mypctile , sortpreserve rclass byable(recall)

	syntax [varlist(default=none)] [if] [in] [, PCTILE(string) BY(varlist) BYDUPL(varlist)]
	
	tokenize `varlist'
	
	local id      "`1'"
	local extvar1 "`2'"
	local pgr     "`3'"
	local pgrstr  "`4'"
	
	tempname  touse 
	mark     `touse' `if' `in'	
	
	qui {
	* ergaenze Liste der Perzentile um das maximum (100)
	local anzpctile : word count `pctile'
	local pctile100 = "`pctile' 100"
	tokenize "`pctile100'"
	local pctilelist `*'
	local minpctile : word 1 of `pctilelist'
	local pstring = "min"

	cap label drop pgrforerr
	label define   pgrforerr 0 "default"	
		
	tempvar n nperc hv0 hv1 hv2 maxn xsm extvar_nvalid ndupl minnperc
		
	if "`bydupl'"=="" {
		if "`by'"=="" {
			sort `extvar1' `id', stable		
			generate  `n' = sum(1)	                 if `extvar1'!=. & `touse'
		}
		else if "`by'"!="" { 	
			sort   `by' `extvar1' `id', stable		
			bysort `by': generate  `n' = sum(1)      if `extvar1'!=. & `touse'
		}
	}
	else if "`bydupl'"!="" {
		generate `extvar_nvalid'  = (`extvar1'!=. & `touse')						
		egen     `ndupl'          = total(`extvar_nvalid'), by(`bydupl')
		if "`by'"=="" {		
		    sort   `extvar1' `id', stable				
		    generate `n' = sum(1/`ndupl')
		}
		else if "`by'"!="" { 			
		    sort   `by' `extvar1' `id', stable				
		    bysort `by': generate `n' = sum(1/`ndupl')
		}
*		noi list `id' `extvar1' `ndupl'	if `id'==21 & `ndupl' !=1	
		replace  `extvar1' = `extvar1'/`ndupl'		
	}
	egen	 `maxn'     = max(`n')       if `extvar1'!=., by(`by')   
	generate `nperc'    = `n' / `maxn'	    	
	egen	 `minnperc' = min(`nperc')   if `extvar1'!=., by(`by')   
	
	* xsm is (`extvar'[_n]+`extvar'[_n+1])/2 if n x pct is integer
	generate `xsm'		  = `extvar1'    	         if `extvar1'!=. & `touse'
	
	* W_(n-1)==P; since W_(n)>W_(n-1) n smallest n with W_(n)>P  
	
	* percentiles
	local i = 1
	foreach pct in `pctilelist' {
		replace `xsm' = (`xsm'[_n]+`xsm'[_n+1])/2 if abs(`nperc'-`pct'/100)<.0000001 & `n'!=`maxn'
		local i = `i' + 1
	}
	* categories
	local i = 1
	foreach pct in `pctilelist' {
        replace `pgr' = `i'                   if `pgr'==0 & `nperc'<=(`pct'/100 - .0000001)
		local i = `i' + 1
	}	
    ** kleinstes Perzentil zunächste "nicht" besetzt, deshalb kleinster Wert
	replace `pgr'  = 1                        if `minnperc'>`minpctile'/100 & `nperc'==`minnperc'
	** For maximum extvar pgr==0. Set pgr to largest category : i-1
	replace  `pgr' = `i'-1 				      if `pgr'==0 
	generate `hv0' = (`pgr'==`pgr'[_n-1]+1) 
	* `hv0'==1 & `n'!=`maxn': these are the precentiles; move one observation back
	replace `pgr'    = `pgr'-1                if `hv0'==1 & `n'!=`maxn'    /* nummerisch */
	if "`bydupl'"!="" {	
		egen    `hv2'    = min(`pgr'), by(`id' `xsm')
		replace `pgr'    = `hv2' 
	}	
    * value labels
	local i = 1
	foreach pct in `pctilelist' {
		label define pgrforerr `i' "p`pct'", add
		local i = `i' + 1
	}	
	local i    = `i'-1
	label define pgrforerr `i' "max", modify
    * text 	
	local i = 1
	foreach pct in `pctilelist' {
		local pstring = "`pstring'"+"<=p`pct'" 
		local i = `i' + 1
	}	

	* maximum may be equal for consecutive `pgr's  ->  `pgrstr' is shorter then
	egen    `hv1'    = max(`xsm'), by(`by' `pgr') 
	replace `pgrstr' = `hv1'                         /* percentiles */
	label values `pgr' pgrforerr	
	return local pstring = subinstr("`pstring'","p100","max",1)
	}
end	
