{smcl}

{title:Title}

{phang}
{bf:nobsdes6} {hline 2} Disclosure control for tables, graphs and aggregation 
according to the rules of the Bundesbank RDSC and Destatis (at least 6 observations)

{marker syntax}{...}
{title:Syntax}

{p 8 17 2}
{cmdab:nobsdes6}
{it:identifier } [({it:dupl})] {it:variable}
{ifin}
{opt , by(varlist)}
{opt byest(varname)}
{opt byregio(varlist)}
{opt notab}
{opt generate}
{opt name(varname)}
{opt replace}
{opt miss(string)}
{opt pctile(string)}
{opt histogram(string)}

{opt end(string)}

{marker description}{...}
{title:Description}

{pstd}
{cmd:nobsdes6} replaces {cmd:nobsdes} which is not valid anymore since disclosure
 control rules have changed.
{smcl}
{pstd}
The command {cmd:nobsdes6} followed by the name of an identifier and the name of
 one or more variables reports the number of distinct values for the identifier 
 for non-missing values of the variables. It is a shortcut for {bf:nobsdes6 identifier if variable!=. & identifier!=""}
 (or {identifier!=.}). With {bf:by} it does the same showing a table unless 
 {bf:notab} is specified. If {bf:generate} is specified, a new variable 
 {bf:tg_identifier_variablename} is created. It is 1 for each identifier in each 
 group defined by the levels of the {bf:by} variable and otherwise zero. An own 
 variable name can be provided by the {bf:name} option. 
{smcl}
{pstd}
At the same time {bf:nobsdes6} calculates whether the share of the two largest 
identifiers in the sum of the variables (their absolutes) (concentration ratio 2)
is higher than 85% meaning these two being dominant. {bf:generate} creates 
{bf:cr2_identifier_variablename}. This name cannot be user specified.
{smcl}
{pstd}
The command is useful for exploring data where the records refer to more than 
one level, for example longitudinal data where each record refers to foreign 
direct investments by domestic firms.
{smcl}
{pstd}
If data is aggregated for further use, it is also useful to document that the 
rules of the Bundesbank RDSC are not violated.
{smcl}
{pstd}
It is possible to specify only the identifier. Then the number of distinct 
non-missing values for the identifier is reported.
{smcl}

{title:Options for use with {cmd:nobsdes6}}

{phang}
{cmd:by(}{it:varlist}{cmd:)} provides the names of one or more deterministic 
 categorical variable(s). If one of these variables denotes region, you have to 
 use the {bf:byregio} option. Do not use the {bf:by} option and the {bf:byregio}
 option together.

{phang}
{cmd:byest(}{it:varlist}{cmd:)} provides the name of one or more ordered categorical
 variables that are estimated as for example the bins in a histogram. {bf:byest} 
 is less restrictive than the {bf:by} option. It counts the number of unique 
 identifiers and verifies the dominance criterion in the first category and in 
 the last. For the other categories in case of less than the required number of 
 distinct identifiers it reviews whether at least 15 percent of the variable of 
 at least one identifier lie outside the suspect range. Do not use the 
 {bf:by} option or the {bf:byregio} option and the {bf:byest} option together. 

{phang}
{cmd:byregio(}{it:varlist}{cmd:)} is {error: obligatory} in case of publication
 of results on regional level because requirements are stricter.

{phang}
{cmd:notab} supresses the table of the {bf:by} option, of the {bf:byest} option,
 and of the {bf:percentile} option. It is rarely used with the {bf:by} option 
 because in general you have to display the table.

{phang}
{cmd:name(}{it:varname}{cmd:)} supplies the name of the new variable which 
 contains the number of distinct values of the varlist by levels defined by the 
 categorical variable supplied in the {bf:by}.

{phang}
{cmd:generate} does the same as the {bf:name(}{it:varname}{cmd:)} option but 
 uses as name the variable name with tg_ and cr2_ in front.

{phang}
{cmd:replace} overwrites the variable specified by {bf:name(}{it:varname}{cmd:)}.

{phang}
{cmd:miss()} allows the specification of a value treated as (additional) missing
 e.g. 0 and is {error: obligatory} for continuous variables with the exception of 
 percentiles (and histograms).

{phang}
{cmd:pctile()} allows the specification of percentiles. Ascending order is 
 required. {cmd:nobsdes6} counts the number of unique identifiers and verifies the 
 dominance criterion between the minimum and the first percentile and the highest
 percentile and the maximum. For the ranges defined by the other percentiles it 
 reviews whether in case of less than the required number of distinct identifiers 
 at least 15 percent of the variable of at least one identifier lie outside the 
 suspect range. In case of several variables and different percentiles "|" has 
 to be used as delimiter. If only one set of percentiles is specified this is
 applied for each variable.

{phang}
{cmd:histogram()} enumerates the distinct identifiers in each bin of a histogram.
 Implies {opt notab}. The number of bins has to be specified in brackets. In case 
 of several variables a specific number of bins for each variable can be specified. 
 If only one number of bins is specified this applies for each histogram.

{phang}
{cmd:end()} specifies an end mark to the nobsdes6 output. The file nobsclean.do 
 (available from the RDSC homepage) then erases the text between 
 {it:nobsdes6 ... endmark}. It is also possible to specify the global macro 
 global endmark {it:"endmark"} to erase all {it:nobs .... endmark}  output.


{marker reserved}{...}
{title:Reserved words}

{bf:pgrforerr} name of label


{marker examples}{...}
{title:Examples}

1) Consider a longitudinal data set in which each record corresponds to a 
foreign direct investment (nu2) by a domestic firm (num). The subject identity 
is in the variable num, and the direct investment is identified by the variable 
nu2, within num. Then

{bf:nobsdes6} num nu2

reports the number of different domestic firms with foreign direct investments.

{bf:nobsdes6} nu2 

reports the number of different foreign direct investments.

{bf:nobsdes6} num nu2, by(lan) gen

creates a new variable {bf:tg_num_nu2} at the subject level which contains the 
number of different domestic firms with foreign direct investments for each country. 

2) Random data:

A do-file with the code:
{txt}
{com}clear
{com}set obs 100
{com}generate id = int(_n/10)+1
{com}generate time = _n-10*(id-1)+1
{com}generate y = runiform()
{com}generate x = runiform()
{com}generate z = runiform()
{com}generate dum_2ids = _n==5|_n==57
{com}generate dum_5ids = _n==5|_n==57|_n==63|_n==99|_n==178
{com}generate dum3 = id==1
{com}replace x = . if id==11
{com}replace z = 0 if id!=1
{txt}
{com}global idlist "id"
{com}nobsdes6 id x
{txt}
{com}. nobsdes6 id x

Output control
No problem with dominance and sufficient number of distinct IDs (id) (=nobs) of 
variable:
                                                          nobs
x             -                                             10

{txt}
{com}nobsdes6 id x y
{txt}
{com}. nobsdes6 id x y

Output control
No problem with dominance and sufficient number of distinct IDs (id) (=nobs) of 
variable:
                                                          nobs
x             -                                             10
y             -                                             11

{com}. nobsdes6 id dum3
dum3 is a 0-1 dummy variable. Dummy and categorial variables must not be part of varlist but must be included in by()

Output control

D I S C L O S U R E problem: 
dum3          -                                              Share of largest two IDs > 85% 

{txt}
3) Table with random data:

{com}nobsdes6 id x, by(time) gen notab
{txt}
Output control
No problem with dominance and sufficient number of distinct IDs (id) (=nobs) of 
variable:
                                                          nobs
x             -                                              9

{com}table time, c(total tg_id_x  mean x  mean cr2_id_x)
{txt}
{com}. table time, c(total tg_id_x  mean x  mean cr2_id_x)

{txt}{hline 5}{c TT}{hline 31}
   {txt}  {c |}    Total           Mean
   {txt}  {c |}  tg_id_x          x   cr2_id_x
{txt}{hline 5}{c +}{hline 31}
{txt}time {c |}
  {txt}1  {c |}       {res} 9   .5655813   34.32216
  {txt}2  {c |}       {res}10   .4303508   34.92011
  {txt}3  {c |}       {res}10   .5312195   34.98271
  {txt}4  {c |}       {res}10   .4708712   35.73005
  {txt}5  {c |}       {res}10   .4739591   35.72026
  {txt}6  {c |}       {res}10   .5756027   28.15181
  {txt}7  {c |}       {res}10   .5992752   30.72029
  {txt}8  {c |}       {res}10   .6938413   28.07875
  {txt}9  {c |}       {res}10   .3236031   51.04036
  {txt}10 {c |}       {res}10   .4075139   38.85685
{txt}{hline 5}{c BT}{hline 31}
{txt}
{com}.
{txt}

4) Graph showing the mean of x (save the graph as .png-file)
   (taken from nobsdes6_15xmpl.do, see below):

{com}nobsdes6 id x, by(year land) notab
{com}nobsdes6 id x, by(year land) 
{com}egen meanx = mean(x) if year!=2002, by(year land)
{com}egen tgland = tag(year land)
{com}gsort -tgland year land
{com}line meanx year if land=="BE" & tgland
{com}* graph save graphname.png
{txt}


{marker author}{...}
{title:Author}
		Harald Stahl
		Harald.Stahl@bundesbank.de
        
{marker alsosee}{...}
{title:Also see}

Further examples: 
nobsdes6_15xmpl.do
